"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNameFromPkg = getNameFromPkg;
exports.dependenciesConflictCheck = dependenciesConflictCheck;
exports.parseContentToSingular = parseContentToSingular;
exports.getSingularName = getSingularName;
exports.default = void 0;

var _fs = require("fs");

var _path = require("path");

var _semver = _interopRequireDefault(require("semver"));

var _chalk = _interopRequireDefault(require("chalk"));

var _clipboardy = _interopRequireDefault(require("clipboardy"));

var _constants = require("../../../constants");

var _writeNewRoute = _interopRequireDefault(require("../../../utils/writeNewRoute"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = require('debug')('umi-build-dev:MaterialGenerator');

function getConfigFile(cwd) {
  // TODO maybe add a paths.absConfigPath
  const files = _constants.CONFIG_FILES.map(file => (0, _path.join)(cwd, file)).filter(file => (0, _fs.existsSync)(file));

  return files[0];
}

function getNameFromPkg(pkg) {
  if (!pkg.name) {
    return null;
  }

  return pkg.name.split('/').pop();
}

function dependenciesConflictCheck(blockPkgDeps = {}, projectPkgDeps = {}) {
  const lacks = [];
  const conflicts = [];
  Object.keys(blockPkgDeps).forEach(dep => {
    if (!projectPkgDeps[dep]) {
      lacks.push([dep, blockPkgDeps[dep]]);
    } else if (!_semver.default.intersects(projectPkgDeps[dep], blockPkgDeps[dep])) {
      conflicts.push([dep, blockPkgDeps[dep], projectPkgDeps[dep]]);
    }
  });
  return {
    conflicts,
    lacks
  };
}

const singularReg = new RegExp(`[\'\"](@\/|[\\.\/]+)(${_constants.SINGULAR_SENSLTIVE.join('|')})\/`, 'g');

function parseContentToSingular(content) {
  return content.replace(singularReg, (all, prefix, match) => {
    return all.replace(match, match.replace(/s$/, ''));
  });
}

function getSingularName(name) {
  if (_constants.SINGULAR_SENSLTIVE.includes(name)) {
    name = name.replace(/s$/, '');
  }

  return name;
}

var _default = api => {
  const paths = api.paths,
        log = api.log,
        Generator = api.Generator,
        config = api.config,
        applyPlugins = api.applyPlugins;
  return class MaterialGenerator extends Generator {
    constructor(args, opts) {
      super(args, opts);
      console.log('args', args, opts);
      this.sourcePath = opts.sourcePath;
      this.dryRun = opts.dryRun;
      this.npmClient = opts.npmClient || 'npm';
      this.path = opts.path;
      this.skipDependencies = opts.skipDependencies;
      this.skipModifyRoutes = opts.skipModifyRoutes;
      this.on('error', e => {
        debug(e); // handle the error for aviod throw generator default error stack
      });
      this.on('end', () => {
        const viewUrl = `http://localhost:${process.env.PORT || '8000'}${this.path.toLowerCase()}`;

        if (config.routes && !this.skipModifyRoutes) {
          log.info('start write new route to your routes config...');

          try {
            (0, _writeNewRoute.default)(this.path, getConfigFile(paths.cwd), paths.absSrcPath);
            log.info('write done');
          } catch (e) {
            debug(e);
            log.warn(e.message);
            log.warn('write routes failed, you need modify it by yourself');
          }
        }

        _clipboardy.default.writeSync(viewUrl);

        log.success(`probable url ${_chalk.default.cyan(viewUrl)} ${_chalk.default.dim('(copied to clipboard)')} for view the block.`);
      });
    }

    writing() {
      var _this = this;

      return _asyncToGenerator(function* () {
        // get block package.json data
        const pkgPath = (0, _path.join)(_this.sourcePath, 'package.json');

        if (!(0, _fs.existsSync)(pkgPath)) {
          throw new Error(`not find package.json in ${_this.sourcePath}`);
        } else {
          // eslint-disable-next-line
          _this.pkg = require(pkgPath);
        } // generate block name


        if (!_this.path) {
          const pkgName = getNameFromPkg(_this.pkg);

          if (!pkgName) {
            return log.error("not find name in block's package.json");
          }

          _this.path = `/${pkgName}`;
        } // fix demo => /demo


        if (!/^\//.test(_this.path)) {
          _this.path = `/${_this.path}`;
        } // check dependencies conflict and install dependencies


        if (_this.skipDependencies) {
          log.info('skip dependencies');
        } else {
          // read project package.json
          const projectPkgPath = applyPlugins('_modifyBlockPackageJSONPath', {
            initialValue: (0, _path.join)(paths.cwd, 'package.json')
          });

          if (!(0, _fs.existsSync)(projectPkgPath)) {
            throw new Error(`not find package.json in your project ${paths.cwd}`);
          } // eslint-disable-next-line


          const projectPkg = require(projectPkgPath); // get confilict dependencies and lack dependencies


          const _applyPlugins = applyPlugins('_modifyBlockDependencies', {
            initialValue: dependenciesConflictCheck(_this.pkg.dependencies, projectPkg.dependencies)
          }),
                conflicts = _applyPlugins.conflicts,
                lacks = _applyPlugins.lacks;

          debug(`conflictDeps ${conflicts}, lackDeps ${lacks}`); // find confilict dependencies throw error

          if (conflicts.length) {
            throw new Error(`
  find dependencies conflict between block and your project:
  ${conflicts.map(info => {
              return `* ${info[0]}: ${info[2]}(your project) not compatible with ${info[1]}(block)`;
            }).join('\n')}`);
          } // find lack confilict, auto install


          if (_this.dryRun) {
            log.log('dryRun is true, skip install dependencies');
          } else if (lacks.length) {
            log.info(`install dependencies ${lacks} with ${_this.npmClient}`); // install block dependencies

            _this.scheduleInstallTask(_this.npmClient, lacks.map(dep => `${dep[0]}@${dep[1]}`), {
              save: true
            }, {
              cwd: (0, _path.dirname)(projectPkgPath)
            });
          }
        }

        let targetPath = (0, _path.join)(paths.absPagesPath, _this.path);
        debug(`get targetPath ${targetPath}`);

        if ((0, _fs.existsSync)(targetPath)) {
          _this.path = (yield _this.prompt({
            type: 'input',
            name: 'path',
            message: `path ${_this.path} already exist, please input a new path for it`,
            required: true,
            default: _this.path
          })).path;
          targetPath = (0, _path.join)(paths.absPagesPath, _this.path);
          debug(`targetPath exist get new targetPath ${targetPath}`);
        }

        if (_this.dryRun) {
          log.log('dryRun is true, skip copy files');
          return;
        } // you can find the copy api detail in https://github.com/SBoudrias/mem-fs-editor/blob/master/lib/actions/copy.js


        log.info('start copy block file to your project...');
        ['src', '@'].forEach(folder => {
          const folderPath = (0, _path.join)(_this.sourcePath, folder);
          const targetFolder = folder === 'src' ? targetPath : paths.absSrcPath;
          const options = {
            process(content) {
              content = String(content);

              if (config.singular) {
                content = parseContentToSingular(content);
              }

              return applyPlugins('_modifyBlockFile', {
                initialValue: content
              });
            }

          };

          if ((0, _fs.existsSync)(folderPath)) {
            (0, _fs.readdirSync)(folderPath).forEach(name => {
              const thePath = (0, _path.join)(folderPath, name);

              if ((0, _fs.statSync)(thePath).isDirectory() && config.singular) {
                // @/components/ => @/src/component/ and ./components/ => ./component etc.
                name = getSingularName(name);
              }

              const realTarget = applyPlugins('_modifyBlockTarget', {
                initialValue: (0, _path.join)(targetFolder, name),
                args: {
                  source: thePath,
                  blockPath: _this.path,
                  sourceName: name
                }
              });
              debug(`copy ${thePath} to ${realTarget}`);

              _this.fs.copy(thePath, realTarget, options);
            });
          }
        });
      })();
    }

  };
};

exports.default = _default;