"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = require("fs");

var _path = require("path");

var _download = require("./download");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('umi-build-dev:blockPlugin');

var _default = api => {
  const log = api.log,
        paths = api.paths;

  function generate(args = {}) {
    try {
      console.log('_', typeof args._);
      const url = args._[0];
      (0, _assert.default)(url, `run ${_chalk.default.cyan.underline('umi help block')} to checkout the usage`);

      const MaterialGenerate = require('./block').default(api);

      debug(`get url ${url}`);
      const sourcePath = (0, _download.getPathWithUrl)(url, log, args);
      const isUserUseYarn = (0, _fs.existsSync)((0, _path.join)(paths.cwd, 'yarn.lock'));

      if (isUserUseYarn) {
        log.log('find yarn.lock in your project, use yarn as the default npm client');
      }

      const defaultNpmClient = isUserUseYarn ? 'yarn' : 'npm';
      const path = args.path,
            _args$npmClient = args.npmClient,
            npmClient = _args$npmClient === void 0 ? defaultNpmClient : _args$npmClient,
            dryRun = args.dryRun,
            skipDependencies = args.skipDependencies,
            skipModifyRoutes = args.skipModifyRoutes;
      debug(`get local sourcePath: ${sourcePath} and npmClient: ${npmClient} and name: ${path}`);
      const generate = new MaterialGenerate(process.argv.slice(4), {
        sourcePath,
        npmClient,
        path,
        dryRun,
        skipDependencies,
        skipModifyRoutes,
        env: {
          cwd: api.cwd
        },
        resolved: __dirname
      });
      generate.run(() => {}).catch(e => {
        debug(e);
        log.error(e.message);
      });
    } catch (e) {
      debug(e);
      log.error(`Use block failed, ${e.message}`);
    }
  }

  const details = `
Examples:

  ${_chalk.default.gray('# get block `demo` which in umi official blocks')}
  umi block https://github.com/umijs/umi-blocks/tree/master/demo

  umi block demo ${_chalk.default.gray('# a shortcut command')}

  umi block demo --path /users/settings/profile ${_chalk.default.gray('# add route to the layout')}
  `.trim();
  api.registerCommand('block', {
    description: 'get block',
    usage: `umi block <a github/gitlab/gitrepo url> [options]`,
    options: {
      '--path': "path name, default is name in block's package.json",
      '--branch': 'git branch, this usually does not need when you use a github url with branch itself',
      '--dry-run': 'for test, block would have done without actually installing and download anything',
      '--npm-client': 'use special npm client, default is npm or yarn(when yarn.lock exist in you project)',
      '--skip-dependencies': 'skip block dependencies install and conflict check',
      '--skip-modify-routes': 'skip modify routes when you use conventional routes'
    },
    details
  }, generate);
};

exports.default = _default;